@@ -2,6 +2,9 @@
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
+ *
+ * Copyright (C) 2022 Sandro Santilli <strk@kbt.io>
+ * Copyright (C) 2022 Martin Davis
  * Copyright 2008 Kevin Neufeld
  *
  * This is free software; you can redistribute and/or modify it under
@@ -10,8 +13,9 @@
  * This program will generate a .png image for every .wkt file specified
  * in this directory's Makefile.  Every .wkt file may contain several
  * entries of geometries represented as WKT strings.  Every line in
- * a wkt file is stylized using a predetermined style (line thinkness,
- * fill color, etc) currently hard coded in this programs main function.
+ * a wkt file is stylized using a predetermined style (line thickness,
+ * fill color, etc).
+ * The styles are specified in the adjacent styles.conf file.
  *
  * In order to generate a png file, ImageMagicK must be installed in the
  * user's path as system calls are invoked to "convert".  In this manner,
@@ -26,6 +30,14 @@
  * rendered outside of the generated image's extents, or may be rendered too
  * small to be recognizable as anything other than a single point.
  *
+ * Usage:
+ *  generator [-v] <source_wktfile> [<output_pngfile>]
+ *
+ * -v - show generated Imagemagick commands
+ *
+ * If <output_pngfile> is omitted the output image PNG file has the
+ * same name as the source file
+ *
  **********************************************************************/
 
 #include <stdio.h>
@@ -33,8 +45,8 @@
 #include <string.h>
 #include <ctype.h>
 #include <sys/wait.h> /* for WEXITSTATUS */
+#include <stdbool.h>
 
-#include "liblwgeom.h"
 #include "liblwgeom_internal.h"
 #include "lwgeom_log.h"
 #include "styles.h"
@@ -43,6 +55,8 @@
 #define MAX_DOUBLE_PRECISION 15
 #define MAX_DIGS_DOUBLE (SHOW_DIGS_DOUBLE + 2) /* +2 for dot and sign */
 
+bool optionVerbose = false;
+
 // Some global styling variables
 char *imageSize = "200x200";
 
@@ -71,7 +85,7 @@ pointarrayToString(char *output, POINTARRAY *pa)
 {
 	char x[OUT_DOUBLE_BUFFER_SIZE];
 	char y[OUT_DOUBLE_BUFFER_SIZE];
-	int i;
+	unsigned int i;
 	char *ptr = output;
 
 	for ( i=0; i < pa->npoints; i++ )
@@ -79,8 +93,8 @@ pointarrayToString(char *output, POINTARRAY *pa)
 		POINT2D pt;
 		getPoint2d_p(pa, i, &pt);
 
-		lwprint_double(pt.x, 10, x, OUT_DOUBLE_BUFFER_SIZE);
-		lwprint_double(pt.y, 10, y, OUT_DOUBLE_BUFFER_SIZE);
+		lwprint_double(pt.x, 10, x);
+		lwprint_double(pt.y, 10, y);
 
 		if ( i ) ptr += sprintf(ptr, " ");
 		ptr += sprintf(ptr, "%s,%s", x, y);
@@ -89,6 +103,93 @@ pointarrayToString(char *output, POINTARRAY *pa)
 	return (ptr - output);
 }
 
+/**
+ * Draws a point in a POINTARRAY to a char* using ImageMagick SVG for styling.
+
+ * @param output a char reference to write the LWPOINT to
+ * @param lwp a reference to a LWPOINT
+ * @return the numbers of character written to *output
+ */
+static size_t
+drawPointSymbol(char *output, POINTARRAY *pa, unsigned int index, int size, char* color)
+{
+	// short-circuit no-op
+	if (size <= 0) return 0;
+
+	char x[OUT_DOUBLE_BUFFER_SIZE];
+	char y1[OUT_DOUBLE_BUFFER_SIZE];
+	char y2[OUT_DOUBLE_BUFFER_SIZE];
+	char *ptr = output;
+
+	POINT2D p;
+	getPoint2d_p(pa, index, &p);
+
+	lwprint_double(p.x, 10, x);
+	lwprint_double(p.y, 10, y1);
+	lwprint_double(p.y + size, 10, y2);
+
+	ptr += sprintf(ptr, "-fill %s -strokewidth 0 ", color);
+	ptr += sprintf(ptr, "-draw \"circle %s,%s %s,%s", x, y1, x, y2);
+	ptr += sprintf(ptr, "'\" ");
+
+	return (ptr - output);
+}
+
+/**
+ * Draws a point in a POINTARRAY to a char* using ImageMagick SVG for styling.
+
+ * @param output a char reference to write the LWPOINT to
+ * @param lwp a reference to a LWPOINT
+ * @return the numbers of character written to *output
+ */
+static size_t
+drawLineArrow(char *output, POINTARRAY *pa, int size, int strokeWidth, char* color)
+{
+	// short-circuit no-op
+	if (size <= 0) return 0;
+	if (pa->npoints <= 1) return 0;
+
+	char s0x[OUT_DOUBLE_BUFFER_SIZE];
+	char s0y[OUT_DOUBLE_BUFFER_SIZE];
+	char s1x[OUT_DOUBLE_BUFFER_SIZE];
+	char s1y[OUT_DOUBLE_BUFFER_SIZE];
+	char s2x[OUT_DOUBLE_BUFFER_SIZE];
+	char s2y[OUT_DOUBLE_BUFFER_SIZE];
+
+	POINT2D pn;
+	getPoint2d_p(pa, pa->npoints-1, &pn);
+	POINT2D pn1;
+	getPoint2d_p(pa, pa->npoints-2, &pn1);
+
+	double dx = pn1.x - pn.x;
+	double dy = pn1.y - pn.y;
+	double len = sqrt(dx*dx + dy*dy);
+	//-- abort if final line segment has length 0
+	if (len <= 0) return 0;
+
+	double offx = -0.5 * size * dy/len;
+	double offy =  0.5 * size * dx/len;
+
+
+	double p1x = pn.x + size * dx / len + offx;
+	double p1y = pn.y + size * dy / len + offy;
+	double p2x = pn.x + size * dx / len - offx;
+	double p2y = pn.y + size * dy / len - offy;
+
+	lwprint_double(pn.x, 10, s0x);
+	lwprint_double(pn.y, 10, s0y);
+	lwprint_double(p1x,  10, s1x);
+	lwprint_double(p1y,  10, s1y);
+	lwprint_double(p2x,  10, s2x);
+	lwprint_double(p2y,  10, s2y);
+
+	char *ptr = output;
+	ptr += sprintf(ptr, "-fill %s -strokewidth %d ", color, 2);
+	ptr += sprintf(ptr, "-draw \"path 'M %s,%s %s,%s %s,%s %s,%s'\" ", s0x, s0y, s1x, s1y, s2x, s2y, s0x, s0y);
+
+	return (ptr - output);
+}
+
 /**
  * Serializes a LWPOINT to a char*.  This is a helper function that partially
  * writes the appropriate draw and fill commands used to generate an SVG image
@@ -112,9 +213,9 @@ drawPoint(char *output, LWPOINT *lwp, LAYERSTYLE *styles)
 	LWDEBUGF(4, "%s", "drawPoint called");
 	LWDEBUGF( 4, "point = %s", lwgeom_to_ewkt((LWGEOM*)lwp) );
 
-	lwprint_double(p.x, 10, x, OUT_DOUBLE_BUFFER_SIZE);
-	lwprint_double(p.y, 10, y1, OUT_DOUBLE_BUFFER_SIZE);
-	lwprint_double(p.y + styles->pointSize, 10, y2, OUT_DOUBLE_BUFFER_SIZE);
+	lwprint_double(p.x, 10, x);
+	lwprint_double(p.y, 10, y1);
+	lwprint_double(p.y + styles->pointSize, 10, y2);
 
 	ptr += sprintf(ptr, "-fill %s -strokewidth 0 ", styles->pointColor);
 	ptr += sprintf(ptr, "-draw \"circle %s,%s %s,%s", x, y1, x, y2);
@@ -145,6 +246,10 @@ drawLineString(char *output, LWLINE *lwl, LAYERSTYLE *style)
 	ptr += pointarrayToString(ptr, lwl->points );
 	ptr += sprintf(ptr, "'\" ");
 
+	ptr += drawPointSymbol(ptr, lwl->points, 0, style->lineStartSize, style->lineColor);
+	ptr += drawPointSymbol(ptr, lwl->points, lwl->points->npoints-1, style->lineEndSize, style->lineColor);
+	ptr += drawLineArrow(ptr, lwl->points, style->lineArrowSize, style->lineWidth, style->lineColor);
+
 	return (ptr - output);
 }
 
@@ -161,7 +266,7 @@ static size_t
 drawPolygon(char *output, LWPOLY *lwp, LAYERSTYLE *style)
 {
 	char *ptr = output;
-	int i;
+	unsigned int i;
 
 	LWDEBUGF(4, "%s", "drawPolygon called");
 	LWDEBUGF( 4, "poly = %s", lwgeom_to_ewkt((LWGEOM*)lwp) );
@@ -192,7 +297,7 @@ static size_t
 drawGeometry(char *output, LWGEOM *lwgeom, LAYERSTYLE *styles )
 {
 	char *ptr = output;
-	int i;
+	unsigned int i;
 	int type = lwgeom->type;
 
 	switch (type)
@@ -220,44 +325,6 @@ drawGeometry(char *output, LWGEOM *lwgeom, LAYERSTYLE *styles )
 	return (ptr - output);
 }
 
-/**
- * Invokes a system call to ImageMagick's "convert" command that adds a drop
- * shadow to the current layer image.
- *
- * @param layerNumber the current working layer number.
- */
-static void
-addDropShadow(int layerNumber)
-{
-	// TODO: change to properly sized string
-	char str[512];
-	sprintf(
-	    str,
-	    "convert tmp%d.png -gravity center \"(\" +clone -background navy -shadow 100x3+4+4 \")\" +swap -background none -flatten tmp%d.png",
-	    layerNumber, layerNumber);
-	LWDEBUGF(4, "%s", str);
-	checked_system(str);
-}
-
-/**
- * Invokes a system call to ImageMagick's "convert" command that adds a
- * highlight to the current layer image.
- *
- * @param layerNumber the current working layer number.
- */
-static void
-addHighlight(int layerNumber)
-{
-	// TODO: change to properly sized string
-	char str[512];
-	sprintf(
-	    str,
-	    "convert tmp%d.png \"(\" +clone -channel A -separate +channel -negate -background black -virtual-pixel background -blur 0x3 -shade 120x55 -contrast-stretch 0%% +sigmoidal-contrast 7x50%% -fill grey50 -colorize 10%% +clone +swap -compose overlay -composite \")\" -compose In -composite tmp%d.png",
-	    layerNumber, layerNumber);
-	LWDEBUGF(4, "%s", str);
-	checked_system(str);
-}
-
 /**
  * Invokes a system call to ImageMagick's "convert" command that reduces
  * the overall filesize
@@ -308,9 +375,7 @@ getStyleName(char **styleName, char* line)
 	char *ptr = strrchr(line, ';');
 	if (ptr == NULL)
 	{
-		*styleName = malloc( 8 );
-		strncpy(*styleName, "Default", 7);
-		(*styleName)[7] = '\0';
+		*styleName = strdup("Default");
 		return 1;
 	}
 	else
@@ -323,39 +388,78 @@ getStyleName(char **styleName, char* line)
 	}
 }
 
+int parseOptions(int argc, const char* argv[] )
+{
+	if (argc <= 1) return 1;
+
+	int argPos = 1;
+	while (strncmp(argv[argPos], "-", 1) == 0) {
+		if (strncmp(argv[argPos], "-v", 2) == 0) {
+			optionVerbose = true;
+		}
+		argPos++;
+	}
+	return argPos;
+}
 
 /**
- * Main Application.  Currently, drawing styles are hardcoded in this method.
- * Future work may entail reading the styles from a .properties file.
+ * Main Application.
  */
 int main( int argc, const char* argv[] )
 {
 	FILE *pfile;
 	LWGEOM *lwgeom;
-	char line [2048];
+	char line [65536];
 	char *filename;
 	int layerCount;
 	LAYERSTYLE *styles;
-	char *image_path = "../images/";
+	char *stylefile_path;
+	const char *image_src;
+	char *ptr;
+	const char *stylefilename = "styles.conf";
 
-	getStyles(&styles);
-
-	if ( argc != 2 || strlen(argv[1]) < 3)
+	int filePos = parseOptions(argc, argv);
+	if ( filePos >= argc || strlen(argv[filePos]) < 3)
 	{
-		lwerror("You must specify a wkt filename to convert, and it must be 3 or more characters long.\n");
+		lwerror("Usage: %s [-v] <source_wktfile> [<output_pngfile>]", argv[0]);
 		return -1;
 	}
 
-	if ( (pfile = fopen(argv[1], "r")) == NULL)
+	image_src = argv[filePos];
+
+	if ( (pfile = fopen(image_src, "r")) == NULL)
 	{
-		perror ( argv[1] );
+		perror ( image_src );
 		return -1;
 	}
 
-	filename = malloc( strlen(argv[1]) + strlen(image_path) + 1 );
-	strcpy( filename, image_path );
-	strncat( filename, argv[1], strlen(argv[1])-3 );
-	strncat( filename, "png", 3 );
+	/* Get style */
+	ptr = rindex( image_src, '/' );
+	if ( ptr ) /* source image file has a slash */
+	{
+		size_t dirname_len = (ptr - image_src);
+		stylefile_path = malloc( strlen(stylefilename) + dirname_len + 2);
+		/* copy the directory name */
+		memcpy(stylefile_path, image_src, dirname_len);
+		sprintf(stylefile_path + dirname_len, "/%s", stylefilename);
+	}
+	else /* source image file has no slash, use CWD */
+	{
+		stylefile_path = strdup(stylefilename);
+	}
+	printf("reading styles from %s\n", stylefile_path);
+	getStyles(stylefile_path, &styles);
+	free(stylefile_path);
+
+	if ( argc - filePos >= 2 )
+	{
+		filename = strdup(argv[filePos + 1]);
+	}
+	else
+	{
+		filename = strdup(image_src);
+		sprintf(filename + strlen(image_src) - 3, "png" );
+	}
 
 	printf( "generating %s\n", filename );
 
@@ -392,13 +496,14 @@ int main( int argc, const char* argv[] )
 
 		ptr += sprintf( ptr, "-flip tmp%d.png", layerCount );
 
-		lwfree( lwgeom );
+		lwgeom_free( lwgeom );
 
 		LWDEBUGF( 4, "%s", output );
+		if (optionVerbose) {
+			puts(output);
+		}
 		checked_system(output);
 
-		addHighlight( layerCount );
-		addDropShadow( layerCount );
 		layerCount++;
 		free(styleName);
 	}
