@@ -226,8 +226,8 @@ IsInStdPortalCache(StdPortalCache *STDCache,  char *lextab, char *gaztab, char *
     for (i=0; i<STD_CACHE_ITEMS; i++) {
         StdCacheItem *ci = &STDCache->StdCache[i];
         if (ci->lextab && !strcmp(ci->lextab, lextab) &&
-            ci->lextab && !strcmp(ci->gaztab, gaztab) &&
-            ci->lextab && !strcmp(ci->rultab, rultab))
+            ci->gaztab && !strcmp(ci->gaztab, gaztab) &&
+            ci->rultab && !strcmp(ci->rultab, rultab))
                 return TRUE;
     }
 
@@ -251,8 +251,8 @@ GetStdFromPortalCache(StdPortalCache *STDCache,  char *lextab, char *gaztab, cha
     for (i=0; i<STD_CACHE_ITEMS; i++) {
         StdCacheItem *ci = &STDCache->StdCache[i];
         if (ci->lextab && !strcmp(ci->lextab, lextab) &&
-            ci->lextab && !strcmp(ci->gaztab, gaztab) &&
-            ci->lextab && !strcmp(ci->rultab, rultab))
+            ci->gaztab && !strcmp(ci->gaztab, gaztab) &&
+            ci->rultab && !strcmp(ci->rultab, rultab))
                 return STDCache->StdCache[i].std;
     }
 
@@ -565,7 +565,7 @@ static int parse_rule(char *buf, int *rule)
     TRGT = DatumGetInt32(binval);
 
 #define GET_TEXT_FROM_TUPLE(TRGT,WHICH) \
-    TRGT = DatumGetCString(SPI_getvalue(tuple, tupdesc, WHICH));
+    TRGT = SPI_getvalue(tuple, tupdesc, WHICH);
 
 
 static int fetch_lex_columns(SPITupleTable *tuptable, lex_columns_t *lex_cols)
@@ -623,7 +623,6 @@ static int load_lex(LEXICON *lex, char *tab)
     char *word;
     char *stdword;
     int token;
-    (void)total_tuples;
 
     DBG("start load_lex\n");
     SET_TIME(t1);
@@ -708,6 +707,9 @@ static int load_lex(LEXICON *lex, char *tab)
     ELAPSED_T(t1, t2);
     DBG("Time to read %i lexicon records: %.1f ms.", total_tuples, elapsed);
 
+    (void) total_tuples; // avoid an unused variable warning.
+                         // must be after declaration to pet -Werror=declaration-after-statement
+
     return 0;
 }
 
@@ -837,6 +839,8 @@ static int load_rules(RULES *rules, char *tab)
     ELAPSED_T(t1, t2);
     DBG("Time to read %i rule records: %.1f ms.", total_tuples, elapsed);
 
+    (void) total_tuples; // avoid an unused variable warning.
+                         // must be after declaration to pet -Werror=declaration-after-statement
     return 0;
 }
 
