@@ -47,6 +47,8 @@
 #include "lwgeodetic.h"
 #include "gserialized2.h"
 
+#include <stddef.h>
+
 /***********************************************************************
 * GSERIALIZED metadata utility functions.
 */
@@ -175,9 +177,8 @@ int gserialized2_is_geodetic(const GSERIALIZED *g)
 
 uint32_t gserialized2_max_header_size(void)
 {
-	static const intptr_t size_of_gserialized_up_to_data = (intptr_t) & ((GSERIALIZED *)NULL)->data;
 	/* GSERIALIZED size + max bbox according gbox_serialized_size (XYZM*2) + extended flags + type */
-	return size_of_gserialized_up_to_data + 8 * sizeof(float) + sizeof(uint64_t) + sizeof(uint32_t);
+	return offsetof(GSERIALIZED, data) + 8 * sizeof(float) + sizeof(uint64_t) + sizeof(uint32_t);
 }
 
 
@@ -306,7 +307,7 @@ gserialized2_hash(const GSERIALIZED *g1)
 	size_t hsz1 = gserialized2_header_size(g1);
 	uint8_t *b1 = (uint8_t *)g1 + hsz1;
 	/* Calculate size of type/coordinate buffer */
-	size_t sz1 = SIZE_GET(g1->size);
+	size_t sz1 = LWSIZE_GET(g1->size);
 	size_t bsz1 = sz1 - hsz1;
 	/* Calculate size of srid/type/coordinate buffer */
 	int32_t srid = gserialized2_get_srid(g1);
@@ -716,6 +717,7 @@ static size_t gserialized2_from_lwpoly_size(const LWPOLY *poly)
 {
 	size_t size = 4; /* Type number. */
 	uint32_t i = 0;
+	const size_t point_size = FLAGS_NDIMS(poly->flags) * sizeof(double);
 
 	assert(poly);
 
@@ -726,7 +728,7 @@ static size_t gserialized2_from_lwpoly_size(const LWPOLY *poly)
 	for (i = 0; i < poly->nrings; i++)
 	{
 		size += 4; /* Number of points in ring. */
-		size += poly->rings[i]->npoints * FLAGS_NDIMS(poly->flags) * sizeof(double);
+		size += poly->rings[i]->npoints * point_size;
 	}
 
 	LWDEBUGF(3, "polygon size = %d", size);
@@ -1130,36 +1132,24 @@ static size_t gserialized2_from_extended_flags(lwflags_t lwflags, uint8_t *buf)
 static size_t gserialized2_from_gbox(const GBOX *gbox, uint8_t *buf)
 {
 	uint8_t *loc = buf;
-	float f;
+	float *f;
+	uint8_t i = 0;
 	size_t return_size;
 
 	assert(buf);
 
-	f = next_float_down(gbox->xmin);
-	memcpy(loc, &f, sizeof(float));
-	loc += sizeof(float);
-
-	f = next_float_up(gbox->xmax);
-	memcpy(loc, &f, sizeof(float));
-	loc += sizeof(float);
-
-	f = next_float_down(gbox->ymin);
-	memcpy(loc, &f, sizeof(float));
-	loc += sizeof(float);
-
-	f = next_float_up(gbox->ymax);
-	memcpy(loc, &f, sizeof(float));
-	loc += sizeof(float);
+	f = (float *)buf;
+	f[i++] = next_float_down(gbox->xmin);
+	f[i++] = next_float_up(gbox->xmax);
+	f[i++] = next_float_down(gbox->ymin);
+	f[i++] = next_float_up(gbox->ymax);
+	loc += 4 * sizeof(float);
 
 	if (FLAGS_GET_GEODETIC(gbox->flags))
 	{
-		f = next_float_down(gbox->zmin);
-		memcpy(loc, &f, sizeof(float));
-		loc += sizeof(float);
-
-		f = next_float_up(gbox->zmax);
-		memcpy(loc, &f, sizeof(float));
-		loc += sizeof(float);
+		f[i++] = next_float_down(gbox->zmin);
+		f[i++] = next_float_up(gbox->zmax);
+		loc += 2 * sizeof(float);
 
 		return_size = (size_t)(loc - buf);
 		LWDEBUGF(4, "returning size %d", return_size);
@@ -1168,25 +1158,16 @@ static size_t gserialized2_from_gbox(const GBOX *gbox, uint8_t *buf)
 
 	if (FLAGS_GET_Z(gbox->flags))
 	{
-		f = next_float_down(gbox->zmin);
-		memcpy(loc, &f, sizeof(float));
-		loc += sizeof(float);
-
-		f = next_float_up(gbox->zmax);
-		memcpy(loc, &f, sizeof(float));
-		loc += sizeof(float);
-
+		f[i++] = next_float_down(gbox->zmin);
+		f[i++] = next_float_up(gbox->zmax);
+		loc += 2 * sizeof(float);
 	}
 
 	if (FLAGS_GET_M(gbox->flags))
 	{
-		f = next_float_down(gbox->mmin);
-		memcpy(loc, &f, sizeof(float));
-		loc += sizeof(float);
-
-		f = next_float_up(gbox->mmax);
-		memcpy(loc, &f, sizeof(float));
-		loc += sizeof(float);
+		f[i++] = next_float_down(gbox->mmin);
+		f[i++] = next_float_up(gbox->mmax);
+		loc += 2 * sizeof(float);
 	}
 	return_size = (size_t)(loc - buf);
 	LWDEBUGF(4, "returning size %d", return_size);
@@ -1227,7 +1208,7 @@ GSERIALIZED* gserialized2_from_lwgeom(LWGEOM *geom, size_t *size)
 	** We are aping PgSQL code here, PostGIS code should use
 	** VARSIZE to set this for real.
 	*/
-	SIZE_SET(g->size, expected_size);
+	LWSIZE_SET(g->size, expected_size);
 	g->gflags = lwflags_get_g2flags(geom->flags);
 
 	/* Move write head past size, srid and flags. */
@@ -1246,12 +1227,7 @@ GSERIALIZED* gserialized2_from_lwgeom(LWGEOM *geom, size_t *size)
 	/* Calculate size as returned by data processing functions. */
 	return_size = ptr - (uint8_t*)g;
 
-	if (expected_size != return_size) /* Uh oh! */
-	{
-		lwerror("Return size (%lu) not equal to expected size (%lu)!", return_size, expected_size);
-		return NULL;
-	}
-
+	assert(expected_size == return_size);
 	if (size) /* Return the output size to the caller if necessary. */
 		*size = return_size;
 
@@ -1264,9 +1240,10 @@ GSERIALIZED* gserialized2_from_lwgeom(LWGEOM *geom, size_t *size)
 * De-serialize GSERIALIZED into an LWGEOM.
 */
 
-static LWGEOM* lwgeom_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size);
+static LWGEOM *lwgeom_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size, int32_t srid);
 
-static LWPOINT* lwpoint_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size)
+static LWPOINT *
+lwpoint_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size, int32_t srid)
 {
 	uint8_t *start_ptr = data_ptr;
 	LWPOINT *point;
@@ -1275,7 +1252,7 @@ static LWPOINT* lwpoint_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lw
 	assert(data_ptr);
 
 	point = (LWPOINT*)lwalloc(sizeof(LWPOINT));
-	point->srid = SRID_UNKNOWN; /* Default */
+	point->srid = srid;
 	point->bbox = NULL;
 	point->type = POINTTYPE;
 	point->flags = lwflags;
@@ -1297,7 +1274,8 @@ static LWPOINT* lwpoint_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lw
 	return point;
 }
 
-static LWLINE* lwline_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size)
+static LWLINE *
+lwline_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size, int32_t srid)
 {
 	uint8_t *start_ptr = data_ptr;
 	LWLINE *line;
@@ -1306,7 +1284,7 @@ static LWLINE* lwline_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwfl
 	assert(data_ptr);
 
 	line = (LWLINE*)lwalloc(sizeof(LWLINE));
-	line->srid = SRID_UNKNOWN; /* Default */
+	line->srid = srid;
 	line->bbox = NULL;
 	line->type = LINETYPE;
 	line->flags = lwflags;
@@ -1329,7 +1307,8 @@ static LWLINE* lwline_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwfl
 	return line;
 }
 
-static LWPOLY* lwpoly_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size)
+static LWPOLY *
+lwpoly_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size, int32_t srid)
 {
 	uint8_t *start_ptr = data_ptr;
 	LWPOLY *poly;
@@ -1340,7 +1319,7 @@ static LWPOLY* lwpoly_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwfl
 	assert(data_ptr);
 
 	poly = (LWPOLY*)lwalloc(sizeof(LWPOLY));
-	poly->srid = SRID_UNKNOWN; /* Default */
+	poly->srid = srid;
 	poly->bbox = NULL;
 	poly->type = POLYGONTYPE;
 	poly->flags = lwflags;
@@ -1386,7 +1365,8 @@ static LWPOLY* lwpoly_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwfl
 	return poly;
 }
 
-static LWTRIANGLE* lwtriangle_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size)
+static LWTRIANGLE *
+lwtriangle_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size, int32_t srid)
 {
 	uint8_t *start_ptr = data_ptr;
 	LWTRIANGLE *triangle;
@@ -1395,7 +1375,7 @@ static LWTRIANGLE* lwtriangle_from_gserialized2_buffer(uint8_t *data_ptr, lwflag
 	assert(data_ptr);
 
 	triangle = (LWTRIANGLE*)lwalloc(sizeof(LWTRIANGLE));
-	triangle->srid = SRID_UNKNOWN; /* Default */
+	triangle->srid = srid; /* Default */
 	triangle->bbox = NULL;
 	triangle->type = TRIANGLETYPE;
 	triangle->flags = lwflags;
@@ -1417,7 +1397,8 @@ static LWTRIANGLE* lwtriangle_from_gserialized2_buffer(uint8_t *data_ptr, lwflag
 	return triangle;
 }
 
-static LWCIRCSTRING* lwcircstring_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size)
+static LWCIRCSTRING *
+lwcircstring_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size, int32_t srid)
 {
 	uint8_t *start_ptr = data_ptr;
 	LWCIRCSTRING *circstring;
@@ -1426,7 +1407,7 @@ static LWCIRCSTRING* lwcircstring_from_gserialized2_buffer(uint8_t *data_ptr, lw
 	assert(data_ptr);
 
 	circstring = (LWCIRCSTRING*)lwalloc(sizeof(LWCIRCSTRING));
-	circstring->srid = SRID_UNKNOWN; /* Default */
+	circstring->srid = srid;
 	circstring->bbox = NULL;
 	circstring->type = CIRCSTRINGTYPE;
 	circstring->flags = lwflags;
@@ -1448,7 +1429,8 @@ static LWCIRCSTRING* lwcircstring_from_gserialized2_buffer(uint8_t *data_ptr, lw
 	return circstring;
 }
 
-static LWCOLLECTION* lwcollection_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size)
+static LWCOLLECTION *
+lwcollection_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *size, int32_t srid)
 {
 	uint32_t type;
 	uint8_t *start_ptr = data_ptr;
@@ -1462,7 +1444,7 @@ static LWCOLLECTION* lwcollection_from_gserialized2_buffer(uint8_t *data_ptr, lw
 	data_ptr += 4; /* Skip past the type. */
 
 	collection = (LWCOLLECTION*)lwalloc(sizeof(LWCOLLECTION));
-	collection->srid = SRID_UNKNOWN; /* Default */
+	collection->srid = srid;
 	collection->bbox = NULL;
 	collection->type = type;
 	collection->flags = lwflags;
@@ -1496,7 +1478,7 @@ static LWCOLLECTION* lwcollection_from_gserialized2_buffer(uint8_t *data_ptr, lw
 			lwfree(collection);
 			return NULL;
 		}
-		collection->geoms[i] = lwgeom_from_gserialized2_buffer(data_ptr, lwflags, &subsize);
+		collection->geoms[i] = lwgeom_from_gserialized2_buffer(data_ptr, lwflags, &subsize, srid);
 		data_ptr += subsize;
 	}
 
@@ -1506,7 +1488,8 @@ static LWCOLLECTION* lwcollection_from_gserialized2_buffer(uint8_t *data_ptr, lw
 	return collection;
 }
 
-LWGEOM* lwgeom_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *g_size)
+LWGEOM *
+lwgeom_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, size_t *g_size, int32_t srid)
 {
 	uint32_t type;
 
@@ -1520,15 +1503,15 @@ LWGEOM* lwgeom_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, si
 	switch (type)
 	{
 	case POINTTYPE:
-		return (LWGEOM *)lwpoint_from_gserialized2_buffer(data_ptr, lwflags, g_size);
+		return (LWGEOM *)lwpoint_from_gserialized2_buffer(data_ptr, lwflags, g_size, srid);
 	case LINETYPE:
-		return (LWGEOM *)lwline_from_gserialized2_buffer(data_ptr, lwflags, g_size);
+		return (LWGEOM *)lwline_from_gserialized2_buffer(data_ptr, lwflags, g_size, srid);
 	case CIRCSTRINGTYPE:
-		return (LWGEOM *)lwcircstring_from_gserialized2_buffer(data_ptr, lwflags, g_size);
+		return (LWGEOM *)lwcircstring_from_gserialized2_buffer(data_ptr, lwflags, g_size, srid);
 	case POLYGONTYPE:
-		return (LWGEOM *)lwpoly_from_gserialized2_buffer(data_ptr, lwflags, g_size);
+		return (LWGEOM *)lwpoly_from_gserialized2_buffer(data_ptr, lwflags, g_size, srid);
 	case TRIANGLETYPE:
-		return (LWGEOM *)lwtriangle_from_gserialized2_buffer(data_ptr, lwflags, g_size);
+		return (LWGEOM *)lwtriangle_from_gserialized2_buffer(data_ptr, lwflags, g_size, srid);
 	case MULTIPOINTTYPE:
 	case MULTILINETYPE:
 	case MULTIPOLYGONTYPE:
@@ -1539,7 +1522,7 @@ LWGEOM* lwgeom_from_gserialized2_buffer(uint8_t *data_ptr, lwflags_t lwflags, si
 	case POLYHEDRALSURFACETYPE:
 	case TINTYPE:
 	case COLLECTIONTYPE:
-		return (LWGEOM *)lwcollection_from_gserialized2_buffer(data_ptr, lwflags, g_size);
+		return (LWGEOM *)lwcollection_from_gserialized2_buffer(data_ptr, lwflags, g_size, srid);
 	default:
 		lwerror("Unknown geometry type: %d - %s", type, lwtype_name(type));
 		return NULL;
@@ -1576,7 +1559,7 @@ LWGEOM* lwgeom_from_gserialized2(const GSERIALIZED *g)
 	if (FLAGS_GET_BBOX(lwflags))
 		data_ptr += gbox_serialized_size(lwflags);
 
-	lwgeom = lwgeom_from_gserialized2_buffer(data_ptr, lwflags, &size);
+	lwgeom = lwgeom_from_gserialized2_buffer(data_ptr, lwflags, &size, srid);
 
 	if (!lwgeom)
 		lwerror("%s: unable create geometry", __func__); /* Ooops! */
@@ -1597,8 +1580,6 @@ LWGEOM* lwgeom_from_gserialized2(const GSERIALIZED *g)
 		lwgeom->bbox = NULL;
 	}
 
-	lwgeom_set_srid(lwgeom, srid);
-
 	return lwgeom;
 }
 
@@ -1637,7 +1618,7 @@ GSERIALIZED* gserialized2_set_gbox(GSERIALIZED *g, GBOX *gbox)
 	*/
 	else
 	{
-		size_t varsize_in = SIZE_GET(g->size);
+		size_t varsize_in = LWSIZE_GET(g->size);
 		size_t varsize_out = varsize_in + box_size;
 		uint8_t *ptr_out, *ptr_in, *ptr;
 		g_out = lwalloc(varsize_out);
@@ -1654,7 +1635,7 @@ GSERIALIZED* gserialized2_set_gbox(GSERIALIZED *g, GBOX *gbox)
 		ptr_out += box_size;
 		memcpy(ptr_out, ptr_in, varsize_in - (ptr_in - ptr));
 		G2FLAGS_SET_BBOX(g_out->gflags, 1);
-		SIZE_SET(g_out->size, varsize_out);
+		LWSIZE_SET(g_out->size, varsize_out);
 	}
 
 	/* Move bounds to nearest float values */
@@ -1690,7 +1671,7 @@ GSERIALIZED* gserialized2_drop_gbox(GSERIALIZED *g)
 {
 	int g_ndims = G2FLAGS_NDIMS_BOX(g->gflags);
 	size_t box_size = 2 * g_ndims * sizeof(float);
-	size_t g_out_size = SIZE_GET(g->size) - box_size;
+	size_t g_out_size = LWSIZE_GET(g->size) - box_size;
 	GSERIALIZED *g_out = lwalloc(g_out_size);
 
 	/* Copy the contents while omitting the box */
@@ -1710,7 +1691,7 @@ GSERIALIZED* gserialized2_drop_gbox(GSERIALIZED *g)
 		/* Copy parts after the box into place */
 		memcpy(outptr, inptr, g_out_size - 8);
 		G2FLAGS_SET_BBOX(g_out->gflags, 0);
-		SIZE_SET(g_out->size, g_out_size);
+		LWSIZE_SET(g_out->size, g_out_size);
 	}
 	/* No box? Nothing to do but copy and return. */
 	else
