@@ -22,23 +22,34 @@
  *
  **********************************************************************/
 
-
 #include "lwgeom_sfcgal.h"
+#include <stdio.h>
 
 static int SFCGAL_type_to_lwgeom_type(sfcgal_geometry_type_t type);
-static POINTARRAY* ptarray_from_SFCGAL(const sfcgal_geometry_t* geom, int force3D);
-static sfcgal_geometry_t* ptarray_to_SFCGAL(const POINTARRAY* pa, int type);
-
-
+static POINTARRAY *ptarray_from_SFCGAL(const sfcgal_geometry_t *geom, int force3D);
+static sfcgal_geometry_t *ptarray_to_SFCGAL(const POINTARRAY *pa, int type);
 
 /* Return SFCGAL version string */
-const char*
+const char *
 lwgeom_sfcgal_version()
 {
-        const char *version = sfcgal_version();
-        return version;
+	const char *version = sfcgal_version();
+	return version;
 }
 
+#define MAX_LENGTH_SFCGAL_FULL_VERSION 50
+/* Return SFCGAL full version string */
+const char *
+lwgeom_sfcgal_full_version()
+{
+#if POSTGIS_SFCGAL_VERSION >= 10400
+	const char *version = sfcgal_full_version();
+#else
+	char *version = (char*)malloc(MAX_LENGTH_SFCGAL_FULL_VERSION);
+	snprintf(version, MAX_LENGTH_SFCGAL_FULL_VERSION, "SFCGAL=\"%s\" CGAL=\"Unknown\" Boost=\"Unknown\"", sfcgal_version());
+#endif
+	return version;
+}
 
 /*
  * Mapping between SFCGAL and PostGIS types
@@ -69,9 +80,9 @@ SFCGAL_type_to_lwgeom_type(sfcgal_geometry_type_t type)
 		return MULTIPOLYGONTYPE;
 
 	case SFCGAL_TYPE_MULTISOLID:
-		return COLLECTIONTYPE;  /* Nota: PolyhedralSurface closed inside
-             				   aim is to use true solid type as soon
-	           			   as available in OGC SFS */
+		return COLLECTIONTYPE; /* Nota: PolyhedralSurface closed inside
+					  aim is to use true solid type as soon
+					  as available in OGC SFS */
 
 	case SFCGAL_TYPE_GEOMETRYCOLLECTION:
 		return COLLECTIONTYPE;
@@ -108,19 +119,18 @@ SFCGAL_type_to_lwgeom_type(sfcgal_geometry_type_t type)
 	}
 }
 
-
 /*
  * Return a PostGIS pointarray from a simple SFCGAL geometry:
  * POINT, LINESTRING or TRIANGLE
  *
  * Throw an error on others types
  */
-static POINTARRAY*
-ptarray_from_SFCGAL(const sfcgal_geometry_t* geom, int want3d)
+static POINTARRAY *
+ptarray_from_SFCGAL(const sfcgal_geometry_t *geom, int want3d)
 {
 	POINT4D point;
 	uint32_t i, npoints;
-	POINTARRAY* pa = NULL;
+	POINTARRAY *pa = NULL;
 
 	assert(geom);
 
@@ -148,7 +158,7 @@ ptarray_from_SFCGAL(const sfcgal_geometry_t* geom, int want3d)
 
 		for (i = 0; i < npoints; i++)
 		{
-			const sfcgal_geometry_t* pt = sfcgal_linestring_point_n(geom, i);
+			const sfcgal_geometry_t *pt = sfcgal_linestring_point_n(geom, i);
 			point.x = sfcgal_point_x(pt);
 			point.y = sfcgal_point_y(pt);
 
@@ -168,11 +178,11 @@ ptarray_from_SFCGAL(const sfcgal_geometry_t* geom, int want3d)
 
 		for (i = 0; i < 4; i++)
 		{
-			const sfcgal_geometry_t* pt = sfcgal_triangle_vertex(geom, (i%3));
+			const sfcgal_geometry_t *pt = sfcgal_triangle_vertex(geom, (i % 3));
 			point.x = sfcgal_point_x(pt);
 			point.y = sfcgal_point_y(pt);
 
-			if ( sfcgal_geometry_is_3d(geom))
+			if (sfcgal_geometry_is_3d(geom))
 				point.z = sfcgal_point_z(pt);
 			else if (want3d)
 				point.z = 0.0;
@@ -190,15 +200,14 @@ ptarray_from_SFCGAL(const sfcgal_geometry_t* geom, int want3d)
 	return pa;
 }
 
-
 /*
  * Convert a PostGIS pointarray to SFCGAL structure
  *
  * Used for simple LWGEOM geometry POINT, LINESTRING, TRIANGLE
  * and POLYGON rings
  */
-static sfcgal_geometry_t*
-ptarray_to_SFCGAL(const POINTARRAY* pa, int type)
+static sfcgal_geometry_t *
+ptarray_to_SFCGAL(const POINTARRAY *pa, int type)
 {
 	POINT3DZ point;
 	int is_3d;
@@ -213,14 +222,16 @@ ptarray_to_SFCGAL(const POINTARRAY* pa, int type)
 	case POINTTYPE:
 	{
 		getPoint3dz_p(pa, 0, &point);
-		if (is_3d) return sfcgal_point_create_from_xyz(point.x, point.y, point.z);
-		else       return sfcgal_point_create_from_xy(point.x, point.y);
+		if (is_3d)
+			return sfcgal_point_create_from_xyz(point.x, point.y, point.z);
+		else
+			return sfcgal_point_create_from_xy(point.x, point.y);
 	}
 	break;
 
 	case LINETYPE:
 	{
-		sfcgal_geometry_t* line = sfcgal_linestring_create();
+		sfcgal_geometry_t *line = sfcgal_linestring_create();
 
 		for (i = 0; i < pa->npoints; i++)
 		{
@@ -228,12 +239,11 @@ ptarray_to_SFCGAL(const POINTARRAY* pa, int type)
 			if (is_3d)
 			{
 				sfcgal_linestring_add_point(line,
-				                            sfcgal_point_create_from_xyz(point.x, point.y, point.z));
+							    sfcgal_point_create_from_xyz(point.x, point.y, point.z));
 			}
 			else
 			{
-				sfcgal_linestring_add_point(line,
-				                            sfcgal_point_create_from_xy(point.x, point.y));
+				sfcgal_linestring_add_point(line, sfcgal_point_create_from_xy(point.x, point.y));
 			}
 		}
 
@@ -243,19 +253,25 @@ ptarray_to_SFCGAL(const POINTARRAY* pa, int type)
 
 	case TRIANGLETYPE:
 	{
-		sfcgal_geometry_t* triangle = sfcgal_triangle_create();
+		sfcgal_geometry_t *triangle = sfcgal_triangle_create();
 
 		getPoint3dz_p(pa, 0, &point);
-		if (is_3d) sfcgal_triangle_set_vertex_from_xyz(triangle, 0, point.x, point.y, point.z);
-		else       sfcgal_triangle_set_vertex_from_xy (triangle, 0, point.x, point.y);
+		if (is_3d)
+			sfcgal_triangle_set_vertex_from_xyz(triangle, 0, point.x, point.y, point.z);
+		else
+			sfcgal_triangle_set_vertex_from_xy(triangle, 0, point.x, point.y);
 
 		getPoint3dz_p(pa, 1, &point);
-		if (is_3d) sfcgal_triangle_set_vertex_from_xyz(triangle, 1, point.x, point.y, point.z);
-		else       sfcgal_triangle_set_vertex_from_xy (triangle, 1, point.x, point.y);
+		if (is_3d)
+			sfcgal_triangle_set_vertex_from_xyz(triangle, 1, point.x, point.y, point.z);
+		else
+			sfcgal_triangle_set_vertex_from_xy(triangle, 1, point.x, point.y);
 
 		getPoint3dz_p(pa, 2, &point);
-		if (is_3d) sfcgal_triangle_set_vertex_from_xyz(triangle, 2, point.x, point.y, point.z);
-		else       sfcgal_triangle_set_vertex_from_xy (triangle, 2, point.x, point.y);
+		if (is_3d)
+			sfcgal_triangle_set_vertex_from_xyz(triangle, 2, point.x, point.y, point.z);
+		else
+			sfcgal_triangle_set_vertex_from_xy(triangle, 2, point.x, point.y);
 
 		return triangle;
 	}
@@ -268,16 +284,15 @@ ptarray_to_SFCGAL(const POINTARRAY* pa, int type)
 	}
 }
 
-
 /*
  * Convert a SFCGAL structure to PostGIS LWGEOM
  *
  * Throws an error on unsupported type
  */
-LWGEOM*
-SFCGAL2LWGEOM(const sfcgal_geometry_t* geom, int force3D, int srid)
+LWGEOM *
+SFCGAL2LWGEOM(const sfcgal_geometry_t *geom, int force3D, int32_t srid)
 {
-	uint32_t ngeoms, nshells, nsolids;
+	uint32_t ngeoms = 0, nshells = 0, nsolids = 0;
 	uint32_t i, j, k;
 	int want3d;
 
@@ -290,43 +305,43 @@ SFCGAL2LWGEOM(const sfcgal_geometry_t* geom, int force3D, int srid)
 	case SFCGAL_TYPE_POINT:
 	{
 		if (sfcgal_geometry_is_empty(geom))
-			return (LWGEOM*) lwpoint_construct_empty(srid, want3d, 0);
+			return (LWGEOM *)lwpoint_construct_empty(srid, want3d, 0);
 
-		POINTARRAY* pa = ptarray_from_SFCGAL(geom, want3d);
-		return (LWGEOM*) lwpoint_construct(srid, NULL, pa);
+		POINTARRAY *pa = ptarray_from_SFCGAL(geom, want3d);
+		return (LWGEOM *)lwpoint_construct(srid, NULL, pa);
 	}
 
 	case SFCGAL_TYPE_LINESTRING:
 	{
 		if (sfcgal_geometry_is_empty(geom))
-			return (LWGEOM*) lwline_construct_empty(srid, want3d, 0);
+			return (LWGEOM *)lwline_construct_empty(srid, want3d, 0);
 
-		POINTARRAY* pa = ptarray_from_SFCGAL(geom, want3d);
-		return (LWGEOM*) lwline_construct(srid, NULL, pa);
+		POINTARRAY *pa = ptarray_from_SFCGAL(geom, want3d);
+		return (LWGEOM *)lwline_construct(srid, NULL, pa);
 	}
 
 	case SFCGAL_TYPE_TRIANGLE:
 	{
 		if (sfcgal_geometry_is_empty(geom))
-			return (LWGEOM*) lwtriangle_construct_empty(srid, want3d, 0);
+			return (LWGEOM *)lwtriangle_construct_empty(srid, want3d, 0);
 
-		POINTARRAY* pa = ptarray_from_SFCGAL(geom, want3d);
-		return (LWGEOM*) lwtriangle_construct(srid, NULL, pa);
+		POINTARRAY *pa = ptarray_from_SFCGAL(geom, want3d);
+		return (LWGEOM *)lwtriangle_construct(srid, NULL, pa);
 	}
 
 	case SFCGAL_TYPE_POLYGON:
 	{
 		if (sfcgal_geometry_is_empty(geom))
-			return (LWGEOM*) lwpoly_construct_empty(srid, want3d, 0);
+			return (LWGEOM *)lwpoly_construct_empty(srid, want3d, 0);
 
 		uint32_t nrings = sfcgal_polygon_num_interior_rings(geom) + 1;
-		POINTARRAY** pa = (POINTARRAY**) lwalloc(sizeof(POINTARRAY*) * nrings);
+		POINTARRAY **pa = (POINTARRAY **)lwalloc(sizeof(POINTARRAY *) * nrings);
 
 		pa[0] = ptarray_from_SFCGAL(sfcgal_polygon_exterior_ring(geom), want3d);
 		for (i = 1; i < nrings; i++)
-			pa[i] = ptarray_from_SFCGAL(sfcgal_polygon_interior_ring_n(geom, i-1), want3d);
+			pa[i] = ptarray_from_SFCGAL(sfcgal_polygon_interior_ring_n(geom, i - 1), want3d);
 
-		return (LWGEOM*) lwpoly_construct(srid, NULL, nrings, pa);
+		return (LWGEOM *)lwpoly_construct(srid, NULL, nrings, pa);
 	}
 
 	case SFCGAL_TYPE_MULTIPOINT:
@@ -336,25 +351,29 @@ SFCGAL2LWGEOM(const sfcgal_geometry_t* geom, int force3D, int srid)
 	case SFCGAL_TYPE_GEOMETRYCOLLECTION:
 	{
 		ngeoms = sfcgal_geometry_collection_num_geometries(geom);
-		LWGEOM** geoms = NULL;
+		LWGEOM **geoms = NULL;
 		if (ngeoms)
 		{
 			nsolids = 0;
-			geoms = (LWGEOM**) lwalloc(sizeof(LWGEOM*) * ngeoms);
+			geoms = (LWGEOM **)lwalloc(sizeof(LWGEOM *) * ngeoms);
 			for (i = 0; i < ngeoms; i++)
 			{
-				const sfcgal_geometry_t* g = sfcgal_geometry_collection_geometry_n(geom, i);
+				const sfcgal_geometry_t *g = sfcgal_geometry_collection_geometry_n(geom, i);
 				geoms[i] = SFCGAL2LWGEOM(g, 0, srid);
-				if ( FLAGS_GET_SOLID( geoms[i]->flags ) ) ++nsolids;
+				if (FLAGS_GET_SOLID(geoms[i]->flags))
+					++nsolids;
 			}
-			geoms = (LWGEOM**) lwrealloc(geoms, sizeof(LWGEOM*) * ngeoms);
+			geoms = (LWGEOM **)lwrealloc(geoms, sizeof(LWGEOM *) * ngeoms);
 		}
-		LWGEOM* rgeom = (LWGEOM*) lwcollection_construct(SFCGAL_type_to_lwgeom_type(
-				sfcgal_geometry_type_id(geom)), srid, NULL, ngeoms, geoms);
-		if ( ngeoms )
+		LWGEOM *rgeom = (LWGEOM *)lwcollection_construct(
+		    SFCGAL_type_to_lwgeom_type(sfcgal_geometry_type_id(geom)), srid, NULL, ngeoms, geoms);
+		if (ngeoms)
 		{
-		        if ( ngeoms == nsolids ) FLAGS_SET_SOLID( rgeom->flags, 1);
-			else if ( nsolids ) lwnotice("SFCGAL2LWGEOM: SOLID in heterogeneous collection will be treated as a POLYHEDRALSURFACETYPE");
+			if (ngeoms == nsolids)
+				FLAGS_SET_SOLID(rgeom->flags, 1);
+			else if (nsolids)
+				lwnotice(
+				    "SFCGAL2LWGEOM: SOLID in heterogeneous collection will be treated as a POLYHEDRALSURFACETYPE");
 		}
 		return rgeom;
 	}
@@ -375,17 +394,17 @@ SFCGAL2LWGEOM(const sfcgal_geometry_t* geom, int force3D, int srid)
 	{
 		ngeoms = sfcgal_polyhedral_surface_num_polygons(geom);
 
-		LWGEOM** geoms = NULL;
+		LWGEOM **geoms = NULL;
 		if (ngeoms)
 		{
-			geoms = (LWGEOM**) lwalloc(sizeof(LWGEOM*) * ngeoms);
+			geoms = (LWGEOM **)lwalloc(sizeof(LWGEOM *) * ngeoms);
 			for (i = 0; i < ngeoms; i++)
 			{
-				const sfcgal_geometry_t* g = sfcgal_polyhedral_surface_polygon_n( geom, i );
+				const sfcgal_geometry_t *g = sfcgal_polyhedral_surface_polygon_n(geom, i);
 				geoms[i] = SFCGAL2LWGEOM(g, 0, srid);
 			}
 		}
-		return (LWGEOM*)lwcollection_construct(POLYHEDRALSURFACETYPE, srid, NULL, ngeoms, geoms);
+		return (LWGEOM *)lwcollection_construct(POLYHEDRALSURFACETYPE, srid, NULL, ngeoms, geoms);
 	}
 
 	/* Solid is map as a closed PolyhedralSurface (for now) */
@@ -396,42 +415,43 @@ SFCGAL2LWGEOM(const sfcgal_geometry_t* geom, int force3D, int srid)
 		for (ngeoms = 0, i = 0; i < nshells; i++)
 			ngeoms += sfcgal_polyhedral_surface_num_polygons(sfcgal_solid_shell_n(geom, i));
 
-		LWGEOM** geoms = 0;
+		LWGEOM **geoms = 0;
 		if (ngeoms)
 		{
-			geoms = (LWGEOM**) lwalloc( sizeof(LWGEOM*) * ngeoms);
-			for (i = 0, k =0 ; i < nshells; i++)
+			geoms = (LWGEOM **)lwalloc(sizeof(LWGEOM *) * ngeoms);
+			for (i = 0, k = 0; i < nshells; i++)
 			{
-				const sfcgal_geometry_t* shell = sfcgal_solid_shell_n(geom, i);
+				const sfcgal_geometry_t *shell = sfcgal_solid_shell_n(geom, i);
 				ngeoms = sfcgal_polyhedral_surface_num_polygons(shell);
 
 				for (j = 0; j < ngeoms; j++)
 				{
-					const sfcgal_geometry_t* g = sfcgal_polyhedral_surface_polygon_n(shell, j);
+					const sfcgal_geometry_t *g = sfcgal_polyhedral_surface_polygon_n(shell, j);
 					geoms[k] = SFCGAL2LWGEOM(g, 1, srid);
 					k++;
 				}
 			}
 		}
-		LWGEOM* rgeom =  (LWGEOM*) lwcollection_construct(POLYHEDRALSURFACETYPE, srid, NULL, ngeoms, geoms);
-		if (ngeoms) FLAGS_SET_SOLID( rgeom->flags, 1);
+		LWGEOM *rgeom = (LWGEOM *)lwcollection_construct(POLYHEDRALSURFACETYPE, srid, NULL, ngeoms, geoms);
+		if (ngeoms)
+			FLAGS_SET_SOLID(rgeom->flags, 1);
 		return rgeom;
 	}
 
 	case SFCGAL_TYPE_TRIANGULATEDSURFACE:
 	{
 		ngeoms = sfcgal_triangulated_surface_num_triangles(geom);
-		LWGEOM** geoms = NULL;
+		LWGEOM **geoms = NULL;
 		if (ngeoms)
 		{
-			geoms = (LWGEOM**) lwalloc(sizeof(LWGEOM*) * ngeoms);
+			geoms = (LWGEOM **)lwalloc(sizeof(LWGEOM *) * ngeoms);
 			for (i = 0; i < ngeoms; i++)
 			{
-				const sfcgal_geometry_t* g = sfcgal_triangulated_surface_triangle_n(geom, i);
+				const sfcgal_geometry_t *g = sfcgal_triangulated_surface_triangle_n(geom, i);
 				geoms[i] = SFCGAL2LWGEOM(g, 0, srid);
 			}
 		}
-		return (LWGEOM*) lwcollection_construct(TINTYPE, srid, NULL, ngeoms, geoms);
+		return (LWGEOM *)lwcollection_construct(TINTYPE, srid, NULL, ngeoms, geoms);
 	}
 
 	default:
@@ -440,12 +460,11 @@ SFCGAL2LWGEOM(const sfcgal_geometry_t* geom, int force3D, int srid)
 	}
 }
 
-
-sfcgal_geometry_t*
-LWGEOM2SFCGAL(const LWGEOM* geom)
+sfcgal_geometry_t *
+LWGEOM2SFCGAL(const LWGEOM *geom)
 {
 	uint32_t i;
-	sfcgal_geometry_t* ret_geom = NULL;
+	sfcgal_geometry_t *ret_geom = NULL;
 
 	assert(geom);
 
@@ -453,8 +472,9 @@ LWGEOM2SFCGAL(const LWGEOM* geom)
 	{
 	case POINTTYPE:
 	{
-		const LWPOINT* lwp = (const LWPOINT*) geom;
-		if (lwgeom_is_empty(geom)) return sfcgal_point_create();
+		const LWPOINT *lwp = (const LWPOINT *)geom;
+		if (lwgeom_is_empty(geom))
+			return sfcgal_point_create();
 
 		return ptarray_to_SFCGAL(lwp->point, POINTTYPE);
 	}
@@ -462,8 +482,9 @@ LWGEOM2SFCGAL(const LWGEOM* geom)
 
 	case LINETYPE:
 	{
-		const LWLINE* line = (const LWLINE*) geom;
-		if (lwgeom_is_empty(geom)) return sfcgal_linestring_create();
+		const LWLINE *line = (const LWLINE *)geom;
+		if (lwgeom_is_empty(geom))
+			return sfcgal_linestring_create();
 
 		return ptarray_to_SFCGAL(line->points, LINETYPE);
 	}
@@ -471,25 +492,27 @@ LWGEOM2SFCGAL(const LWGEOM* geom)
 
 	case TRIANGLETYPE:
 	{
-		const LWTRIANGLE* triangle = (const LWTRIANGLE*) geom;
-		if (lwgeom_is_empty(geom)) return sfcgal_triangle_create();
+		const LWTRIANGLE *triangle = (const LWTRIANGLE *)geom;
+		if (lwgeom_is_empty(geom))
+			return sfcgal_triangle_create();
 		return ptarray_to_SFCGAL(triangle->points, TRIANGLETYPE);
 	}
 	break;
 
 	case POLYGONTYPE:
 	{
-		const LWPOLY* poly = (const LWPOLY*) geom;
+		const LWPOLY *poly = (const LWPOLY *)geom;
 		uint32_t nrings = poly->nrings - 1;
 
-		if (lwgeom_is_empty(geom)) return sfcgal_polygon_create();
+		if (lwgeom_is_empty(geom))
+			return sfcgal_polygon_create();
 
-		sfcgal_geometry_t* exterior_ring = ptarray_to_SFCGAL(poly->rings[0], LINETYPE);
+		sfcgal_geometry_t *exterior_ring = ptarray_to_SFCGAL(poly->rings[0], LINETYPE);
 		ret_geom = sfcgal_polygon_create_from_exterior_ring(exterior_ring);
 
 		for (i = 0; i < nrings; i++)
 		{
-			sfcgal_geometry_t* ring = ptarray_to_SFCGAL(poly->rings[i + 1], LINETYPE);
+			sfcgal_geometry_t *ring = ptarray_to_SFCGAL(poly->rings[i + 1], LINETYPE);
 			sfcgal_polygon_add_interior_ring(ret_geom, ring);
 		}
 		return ret_geom;
@@ -501,15 +524,19 @@ LWGEOM2SFCGAL(const LWGEOM* geom)
 	case MULTIPOLYGONTYPE:
 	case COLLECTIONTYPE:
 	{
-		if (geom->type == MULTIPOINTTYPE)        ret_geom = sfcgal_multi_point_create();
-		else if (geom->type == MULTILINETYPE)    ret_geom = sfcgal_multi_linestring_create();
-		else if (geom->type == MULTIPOLYGONTYPE) ret_geom = sfcgal_multi_polygon_create();
-		else                                     ret_geom = sfcgal_geometry_collection_create();
+		if (geom->type == MULTIPOINTTYPE)
+			ret_geom = sfcgal_multi_point_create();
+		else if (geom->type == MULTILINETYPE)
+			ret_geom = sfcgal_multi_linestring_create();
+		else if (geom->type == MULTIPOLYGONTYPE)
+			ret_geom = sfcgal_multi_polygon_create();
+		else
+			ret_geom = sfcgal_geometry_collection_create();
 
-		const LWCOLLECTION* lwc = (const LWCOLLECTION*)geom;
+		const LWCOLLECTION *lwc = (const LWCOLLECTION *)geom;
 		for (i = 0; i < lwc->ngeoms; i++)
 		{
-			sfcgal_geometry_t* g = LWGEOM2SFCGAL(lwc->geoms[i]);
+			sfcgal_geometry_t *g = LWGEOM2SFCGAL(lwc->geoms[i]);
 			sfcgal_geometry_collection_add_geometry(ret_geom, g);
 		}
 
@@ -519,12 +546,12 @@ LWGEOM2SFCGAL(const LWGEOM* geom)
 
 	case POLYHEDRALSURFACETYPE:
 	{
-		const LWPSURFACE* lwp = (const LWPSURFACE*) geom;
+		const LWPSURFACE *lwp = (const LWPSURFACE *)geom;
 		ret_geom = sfcgal_polyhedral_surface_create();
 
 		for (i = 0; i < lwp->ngeoms; i++)
 		{
-			sfcgal_geometry_t* g = LWGEOM2SFCGAL((const LWGEOM*) lwp->geoms[i]);
+			sfcgal_geometry_t *g = LWGEOM2SFCGAL((const LWGEOM *)lwp->geoms[i]);
 			sfcgal_polyhedral_surface_add_polygon(ret_geom, g);
 		}
 		/* We treat polyhedral surface as the only exterior shell,
@@ -540,12 +567,12 @@ LWGEOM2SFCGAL(const LWGEOM* geom)
 
 	case TINTYPE:
 	{
-		const LWTIN* lwp = (const LWTIN*) geom;
+		const LWTIN *lwp = (const LWTIN *)geom;
 		ret_geom = sfcgal_triangulated_surface_create();
 
 		for (i = 0; i < lwp->ngeoms; i++)
 		{
-			sfcgal_geometry_t* g = LWGEOM2SFCGAL((const LWGEOM*) lwp->geoms[i]);
+			sfcgal_geometry_t *g = LWGEOM2SFCGAL((const LWGEOM *)lwp->geoms[i]);
 			sfcgal_triangulated_surface_add_triangle(ret_geom, g);
 		}
 
@@ -554,26 +581,26 @@ LWGEOM2SFCGAL(const LWGEOM* geom)
 	break;
 
 	default:
-		lwerror("LWGEOM2SFCGAL: Unknown geometry type !");
+		lwerror("LWGEOM2SFCGAL: Unsupported geometry type %s !", lwtype_name(geom->type));
 		return NULL;
 	}
 }
 
-
 /*
  * No Operation SFCGAL function, used (only) for cunit tests
  * Take a PostGIS geometry, send it to SFCGAL and return it unchanged (in theory)
  */
-LWGEOM* lwgeom_sfcgal_noop(const LWGEOM* geom_in)
+LWGEOM *
+lwgeom_sfcgal_noop(const LWGEOM *geom_in)
 {
-	sfcgal_geometry_t* converted;
+	sfcgal_geometry_t *converted;
 
 	assert(geom_in);
 
 	converted = LWGEOM2SFCGAL(geom_in);
 	assert(converted);
 
-	LWGEOM* geom_out = SFCGAL2LWGEOM(converted, 0, SRID_UNKNOWN);
+	LWGEOM *geom_out = SFCGAL2LWGEOM(converted, 0, SRID_UNKNOWN);
 	sfcgal_geometry_delete(converted);
 
 	/* copy SRID (SFCGAL does not store the SRID) */
