@@ -589,7 +589,7 @@ nd_box_init_bounds(ND_BOX *a)
 static void
 nd_box_from_gbox(const GBOX *gbox, ND_BOX *nd_box)
 {
-	int d = 0;
+	volatile int d = 0;
 	POSTGIS_DEBUGF(3, " %s", gbox_to_string(gbox));
 
 	nd_box_init(nd_box);
@@ -725,7 +725,6 @@ nd_box_ratio(const ND_BOX *b1, const ND_BOX *b2, int ndims)
 	bool covered = true;
 	double ivol = 1.0;
 	double vol2 = 1.0;
-	double vol1 = 1.0;
 
 	for ( d = 0 ; d < ndims; d++ )
 	{
@@ -741,19 +740,17 @@ nd_box_ratio(const ND_BOX *b1, const ND_BOX *b2, int ndims)
 
 	for ( d = 0; d < ndims; d++ )
 	{
-		double width1 = b1->max[d] - b1->min[d];
 		double width2 = b2->max[d] - b2->min[d];
 		double imin, imax, iwidth;
 
-		vol1 = vol1 * width1;
-		vol2 = vol2 * width2;
+		vol2 *= width2;
 
 		imin = Max(b1->min[d], b2->min[d]);
 		imax = Min(b1->max[d], b2->max[d]);
 		iwidth = imax - imin;
 		iwidth = Max(0.0, iwidth);
 
-		ivol = ivol * iwidth;
+		ivol *= iwidth;
 	}
 
 	if ( vol2 == 0.0 )
@@ -1390,7 +1387,6 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 	size_t    nd_stats_size;           /* Size to allocate */
 
 	double total_width = 0;            /* # of bytes used by sample */
-	double total_sample_volume = 0;    /* Area/volume coverage of the sample */
 	double total_cell_count = 0;       /* # of cells in histogram affected by sample */
 
 	ND_BOX sum;                        /* Sum of extents of sample boxes */
@@ -1529,7 +1525,11 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 	 * we have an average of 5 features for each cell so the histogram isn't
 	 * so sparse.
 	 */
+#if POSTGIS_PGSQL_VERSION >= 170
+	histo_cells_target = (int)pow((double)(stats->attstattarget), (double)ndims);
+#else
 	histo_cells_target = (int)pow((double)(stats->attr->attstattarget), (double)ndims);
+#endif
 	histo_cells_target = Min(histo_cells_target, ndims * 10000);
 	histo_cells_target = Min(histo_cells_target, (int)(total_rows/5));
 	POSTGIS_DEBUGF(3, " stats->attr->attstattarget: %d", stats->attr->attstattarget);
@@ -1538,16 +1538,6 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 	/* If there's no useful features, we can't work out stats */
 	if ( ! notnull_cnt )
 	{
-#if POSTGIS_DEBUG_LEVEL > 0
-		Oid relation_oid = stats->attr->attrelid;
-		char *relation_name = get_rel_name(relation_oid);
-		char *namespace = get_namespace_name(get_rel_namespace(relation_oid));
-		elog(DEBUG1,
-		     "PostGIS: Unable to compute statistics for \"%s.%s.%s\": No non-null/empty features",
-		     namespace ? namespace : "(NULL)",
-		     relation_name ? relation_name : "(NULL)",
-		     stats->attr->attname.data);
-#endif /* POSTGIS_DEBUG_LEVEL > 0 */
 		stats->stats_valid = false;
 		return;
 	}
@@ -1733,9 +1723,7 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 		const ND_BOX *nd_box;
 		ND_IBOX nd_ibox;
 		int at[ND_DIMS];
-		int d;
 		double num_cells = 0;
-		double tmp_volume = 1.0;
 		double min[ND_DIMS] = {0.0, 0.0, 0.0, 0.0};
 		double max[ND_DIMS] = {0.0, 0.0, 0.0, 0.0};
 		double cellsize[ND_DIMS] = {0.0, 0.0, 0.0, 0.0};
@@ -1761,14 +1749,8 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 			min[d] = nd_stats->extent.min[d];
 			max[d] = nd_stats->extent.max[d];
 			cellsize[d] = (max[d] - min[d])/(nd_stats->size[d]);
-
-			/* What's the volume (area) of this feature's box? */
-			tmp_volume *= (nd_box->max[d] - nd_box->min[d]);
 		}
 
-		/* Add feature volume (area) to our total */
-		total_sample_volume = total_sample_volume + tmp_volume;
-
 		/*
 		 * Move through all the overlaped histogram cells values and
 		 * add the box overlap proportion to them.
