@@ -27,10 +27,11 @@
  *
  */
 
+// For stat64()
+#define _LARGEFILE64_SOURCE 1
+
 #include <postgres.h>
-#include <fmgr.h>
-#include <funcapi.h>
-#include <utils/builtins.h>
+#include <utils/builtins.h> /* for text_to_cstring() */
 #include "utils/lsyscache.h" /* for get_typlenbyvalalign */
 #include "utils/array.h" /* for ArrayType */
 #include "catalog/pg_type.h" /* for INT2OID, INT4OID, FLOAT4OID, FLOAT8OID and TEXTOID */
@@ -43,6 +44,8 @@
 
 #include "rtpostgis.h"
 
+extern bool enable_outdb_rasters;
+
 /* Get all the properties of a raster band */
 Datum RASTER_getBandPixelType(PG_FUNCTION_ARGS);
 Datum RASTER_getBandPixelTypeName(PG_FUNCTION_ARGS);
@@ -56,6 +59,8 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS);
 /* Set all the properties of a raster band */
 Datum RASTER_setBandIsNoData(PG_FUNCTION_ARGS);
 Datum RASTER_setBandNoDataValue(PG_FUNCTION_ARGS);
+Datum RASTER_setBandPath(PG_FUNCTION_ARGS);
+Datum RASTER_setBandIndex(PG_FUNCTION_ARGS);
 
 /**
  * Return pixel type of the specified band of raster.
@@ -329,9 +334,7 @@ Datum RASTER_getBandPath(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
-	result = (text *) palloc(VARHDRSZ + strlen(bandpath) + 1);
-	SET_VARSIZE(result, VARHDRSZ + strlen(bandpath) + 1);
-	strcpy((char *) VARDATA(result), bandpath);
+	result = cstring_to_text(bandpath);
 
 	rt_band_destroy(band);
 	rt_raster_destroy(raster);
@@ -340,6 +343,127 @@ Datum RASTER_getBandPath(PG_FUNCTION_ARGS)
 	PG_RETURN_TEXT_P(result);
 }
 
+
+/**
+ * Return the file size of the raster.
+ */
+PG_FUNCTION_INFO_V1(RASTER_getBandFileSize);
+Datum RASTER_getBandFileSize(PG_FUNCTION_ARGS)
+{
+    rt_pgraster *pgraster;
+    rt_raster raster;
+    rt_band band = NULL;
+    int64_t fileSize;
+    int32_t bandindex;
+
+    /* Index is 1-based */
+    bandindex = PG_GETARG_INT32(1);
+    if ( bandindex < 1 ) {
+            elog(NOTICE, "Invalid band index (must use 1-based). Returning NULL");
+            PG_RETURN_NULL();
+    }
+
+    if (PG_ARGISNULL(0)) PG_RETURN_NULL();
+    pgraster = (rt_pgraster *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+
+    raster = rt_raster_deserialize(pgraster, FALSE);
+    if ( ! raster ) {
+        PG_FREE_IF_COPY(pgraster, 0);
+        elog(ERROR, "RASTER_getFileSize: Could not deserialize raster");
+        PG_RETURN_NULL();
+    }
+
+    /* Fetch requested band */
+    band = rt_raster_get_band(raster, bandindex - 1);
+    if (!band) {
+        elog(
+                NOTICE,
+                "Could not find raster band of index %d when getting band path. Returning NULL",
+                bandindex
+        );
+        rt_raster_destroy(raster);
+        PG_FREE_IF_COPY(pgraster, 0);
+        PG_RETURN_NULL();
+    }
+
+    if (!rt_band_is_offline(band)) {
+        elog(NOTICE, "Band of index %d is not out-db.", bandindex);
+        rt_band_destroy(band);
+        rt_raster_destroy(raster);
+        PG_FREE_IF_COPY(pgraster, 0);
+        PG_RETURN_NULL();
+    }
+
+    fileSize = rt_band_get_file_size(band);
+
+    rt_band_destroy(band);
+    rt_raster_destroy(raster);
+    PG_FREE_IF_COPY(pgraster, 0);
+
+    PG_RETURN_INT64(fileSize);
+}
+
+/**
+ * Return the file timestamp of the raster.
+ */
+PG_FUNCTION_INFO_V1(RASTER_getBandFileTimestamp);
+Datum RASTER_getBandFileTimestamp(PG_FUNCTION_ARGS)
+{
+    rt_pgraster *pgraster;
+    rt_raster raster;
+    rt_band band = NULL;
+    int64_t fileSize;
+    int32_t bandindex;
+
+    /* Index is 1-based */
+    bandindex = PG_GETARG_INT32(1);
+    if ( bandindex < 1 ) {
+            elog(NOTICE, "Invalid band index (must use 1-based). Returning NULL");
+            PG_RETURN_NULL();
+    }
+
+    if (PG_ARGISNULL(0)) PG_RETURN_NULL();
+    pgraster = (rt_pgraster *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+
+    raster = rt_raster_deserialize(pgraster, FALSE);
+    if ( ! raster ) {
+        PG_FREE_IF_COPY(pgraster, 0);
+        elog(ERROR, "RASTER_getBandFileTimestamp: Could not deserialize raster");
+        PG_RETURN_NULL();
+    }
+
+    /* Fetch requested band */
+    band = rt_raster_get_band(raster, bandindex - 1);
+    if (!band) {
+        elog(
+                NOTICE,
+                "Could not find raster band of index %d when getting band path. Returning NULL",
+                bandindex
+        );
+        rt_raster_destroy(raster);
+        PG_FREE_IF_COPY(pgraster, 0);
+        PG_RETURN_NULL();
+    }
+
+    if (!rt_band_is_offline(band)) {
+        elog(NOTICE, "Band of index %d is not out-db.", bandindex);
+        rt_band_destroy(band);
+        rt_raster_destroy(raster);
+        PG_FREE_IF_COPY(pgraster, 0);
+        PG_RETURN_NULL();
+    }
+
+    fileSize = rt_band_get_file_timestamp(band);
+
+    rt_band_destroy(band);
+    rt_raster_destroy(raster);
+    PG_FREE_IF_COPY(pgraster, 0);
+
+    PG_RETURN_INT64(fileSize);
+}
+
+#define VALUES_LENGTH 8
+
 /**
  * Get raster bands' meta data
  */
@@ -358,6 +482,10 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 		double nodataval;
 		bool isoutdb;
 		char *bandpath;
+		uint8_t extbandnum;
+		uint64_t filesize;
+		uint64_t timestamp;
+		bool isnullband;
 	};
 	struct bandmetadata *bmd = NULL;
 	struct bandmetadata *bmd2 = NULL;
@@ -386,7 +514,9 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 		uint32_t numBands;
 		uint32_t idx = 1;
 		uint32_t *bandNums = NULL;
-		const char *tmp = NULL;
+		const char *chartmp = NULL;
+		size_t charlen;
+		uint8_t extbandnum;
 
 		POSTGIS_RT_DEBUG(3, "RASTER_bandmetadata: Starting");
 
@@ -396,10 +526,29 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 		/* switch to memory context appropriate for multiple function calls */
 		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
 
+		/* Build a tuple descriptor for our result type */
+		if (get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE) {
+			MemoryContextSwitchTo(oldcontext);
+			ereport(ERROR, (
+				errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				errmsg(
+					"function returning record called in context "
+					"that cannot accept type record"
+				)
+			));
+		}
+
+		BlessTupleDesc(tupdesc);
+		funcctx->tuple_desc = tupdesc;
+
 		/* pgraster is null, return null */
 		if (PG_ARGISNULL(0)) {
+			bmd = (struct bandmetadata *) palloc(sizeof(struct bandmetadata));
+			bmd->isnullband = TRUE;
+			funcctx->user_fctx = bmd;
+			funcctx->max_calls = 1;
 			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
+			goto PER_CALL;
 		}
 		pgraster = (rt_pgraster *) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
 
@@ -418,8 +567,12 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 			elog(NOTICE, "Raster provided has no bands");
 			rt_raster_destroy(raster);
 			PG_FREE_IF_COPY(pgraster, 0);
+			bmd = (struct bandmetadata *) palloc(sizeof(struct bandmetadata));
+			bmd->isnullband = TRUE;
+			funcctx->user_fctx = bmd;
+			funcctx->max_calls = 1;
 			MemoryContextSwitchTo(oldcontext);
-			SRF_RETURN_DONE(funcctx);
+			goto PER_CALL;
 		}
 
 		/* band index */
@@ -462,8 +615,12 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 				pfree(bandNums);
 				rt_raster_destroy(raster);
 				PG_FREE_IF_COPY(pgraster, 0);
+				bmd = (struct bandmetadata *) palloc(sizeof(struct bandmetadata));
+				bmd->isnullband = TRUE;
+				funcctx->user_fctx = bmd;
+				funcctx->max_calls = 1;
 				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
+				goto PER_CALL;
 			}
 
 			bandNums[j] = idx;
@@ -480,7 +637,7 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 		else if (j < n)
 			bandNums = repalloc(bandNums, sizeof(uint32_t) * j);
 
-		bmd = (struct bandmetadata *) palloc(sizeof(struct bandmetadata) * j);
+		bmd = (struct bandmetadata *) palloc0(sizeof(struct bandmetadata) * j);
 
 		for (i = 0; i < j; i++) {
 			band = rt_raster_get_band(raster, bandNums[i] - 1);
@@ -488,17 +645,21 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 				elog(NOTICE, "Could not get raster band at index %d", bandNums[i]);
 				rt_raster_destroy(raster);
 				PG_FREE_IF_COPY(pgraster, 0);
+				bmd[0].isnullband = TRUE;
+				funcctx->user_fctx = bmd;
+				funcctx->max_calls = 1;
 				MemoryContextSwitchTo(oldcontext);
-				SRF_RETURN_DONE(funcctx);
+				goto PER_CALL;
 			}
 
 			/* bandnum */
 			bmd[i].bandnum = bandNums[i];
 
 			/* pixeltype */
-			tmp = rt_pixtype_name(rt_band_get_pixtype(band));
-			bmd[i].pixeltype = palloc(sizeof(char) * (strlen(tmp) + 1));
-			strncpy(bmd[i].pixeltype, tmp, strlen(tmp) + 1);
+			chartmp = rt_pixtype_name(rt_band_get_pixtype(band));
+			charlen = strlen(chartmp) + 1;
+			bmd[i].pixeltype = palloc(sizeof(char) * charlen);
+			strncpy(bmd[i].pixeltype, chartmp, charlen);
 
 			/* hasnodatavalue */
 			if (rt_band_get_hasnodata_flag(band))
@@ -512,11 +673,12 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 			else
 				bmd[i].nodataval = 0;
 
-			/* path */
-			tmp = rt_band_get_ext_path(band);
-			if (tmp) {
-				bmd[i].bandpath = palloc(sizeof(char) * (strlen(tmp) + 1));
-				strncpy(bmd[i].bandpath, tmp, strlen(tmp) + 1);
+			/* out-db path */
+			chartmp = rt_band_get_ext_path(band);
+			if (chartmp) {
+				charlen = strlen(chartmp) + 1;
+				bmd[i].bandpath = palloc(sizeof(char) * charlen);
+				strncpy(bmd[i].bandpath, chartmp, charlen);
 			}
 			else
 				bmd[i].bandpath = NULL;
@@ -524,6 +686,22 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 			/* isoutdb */
 			bmd[i].isoutdb = bmd[i].bandpath ? TRUE : FALSE;
 
+			/* out-db bandnum */
+			if (rt_band_get_ext_band_num(band, &extbandnum) == ES_NONE)
+				bmd[i].extbandnum = extbandnum + 1;
+			else
+				bmd[i].extbandnum = 0;
+
+                        bmd[i].filesize = 0;
+                        bmd[i].timestamp = 0;
+                        if( bmd[i].bandpath && enable_outdb_rasters ) {
+                            VSIStatBufL sStat;
+                            if( VSIStatL(bmd[i].bandpath, &sStat) == 0 ) {
+                                bmd[i].filesize = sStat.st_size;
+                                bmd[i].timestamp = sStat.st_mtime;
+                            }
+                        }
+
 			rt_band_destroy(band);
 		}
 
@@ -536,24 +714,11 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 		/* total number of tuples to be returned */
 		funcctx->max_calls = j;
 
-		/* Build a tuple descriptor for our result type */
-		if (get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE) {
-			MemoryContextSwitchTo(oldcontext);
-			ereport(ERROR, (
-				errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				errmsg(
-					"function returning record called in context "
-					"that cannot accept type record"
-				)
-			));
-		}
-
-		BlessTupleDesc(tupdesc);
-		funcctx->tuple_desc = tupdesc;
-
 		MemoryContextSwitchTo(oldcontext);
 	}
 
+	PER_CALL:
+
 	/* stuff done on every call of the function */
 	funcctx = SRF_PERCALL_SETUP();
 
@@ -564,11 +729,18 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 
 	/* do when there is more left to send */
 	if (call_cntr < max_calls) {
-		int values_length = 5;
-		Datum values[values_length];
-		bool nulls[values_length];
+		Datum values[VALUES_LENGTH];
+		bool nulls[VALUES_LENGTH];
 
-		memset(nulls, FALSE, sizeof(bool) * values_length);
+		if (bmd2[0].isnullband) {
+			int i;
+			for (i = 0; i < VALUES_LENGTH; i++)
+				nulls[i] = TRUE;
+			result = HeapTupleGetDatum(heap_form_tuple(tupdesc, values, nulls));
+			SRF_RETURN_NEXT(funcctx, result);
+		}
+
+		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 		values[0] = UInt32GetDatum(bmd2[call_cntr].bandnum);
 		values[1] = CStringGetTextDatum(bmd2[call_cntr].pixeltype);
@@ -579,10 +751,25 @@ Datum RASTER_bandmetadata(PG_FUNCTION_ARGS)
 			nulls[2] = TRUE;
 
 		values[3] = BoolGetDatum(bmd2[call_cntr].isoutdb);
-		if (bmd2[call_cntr].bandpath && strlen(bmd2[call_cntr].bandpath))
+		if (bmd2[call_cntr].bandpath && strlen(bmd2[call_cntr].bandpath)) {
 			values[4] = CStringGetTextDatum(bmd2[call_cntr].bandpath);
-		else
+			values[5] = UInt32GetDatum(bmd2[call_cntr].extbandnum);
+		}
+		else {
 			nulls[4] = TRUE;
+			nulls[5] = TRUE;
+		}
+
+		if (bmd2[call_cntr].filesize)
+		{
+			values[6] = UInt64GetDatum(bmd2[call_cntr].filesize);
+			values[7] = UInt64GetDatum(bmd2[call_cntr].timestamp);
+		}
+		else
+		{
+			nulls[6] = TRUE;
+			nulls[7] = TRUE;
+		}
 
 		/* build a tuple */
 		tuple = heap_form_tuple(tupdesc, values, nulls);
@@ -679,6 +866,9 @@ Datum RASTER_setBandNoDataValue(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(pgrtn);
 }
 
+/**
+ * Set flag indicating that the entire band is NODATA
+ */
 PG_FUNCTION_INFO_V1(RASTER_setBandIsNoData);
 Datum RASTER_setBandIsNoData(PG_FUNCTION_ARGS)
 {
@@ -734,3 +924,94 @@ Datum RASTER_setBandIsNoData(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(pgrtn);
 }
 
+/**
+ * Set the path value of an out-db band
+ */
+PG_FUNCTION_INFO_V1(RASTER_setBandPath);
+Datum RASTER_setBandPath(PG_FUNCTION_ARGS)
+{
+	rt_pgraster *pgraster = NULL;
+	rt_pgraster *pgrtn = NULL;
+	rt_raster raster = NULL;
+	rt_band band = NULL;
+	int32_t bandindex = 1;
+	const char *outdbpathchar = NULL;
+	int32_t outdbindex = 1;
+	bool forceset = FALSE;
+	rt_band newband = NULL;
+
+	int hasnodata;
+	double nodataval = 0.;
+
+	if (PG_ARGISNULL(0))
+		PG_RETURN_NULL();
+	pgraster = (rt_pgraster *) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+
+	raster = rt_raster_deserialize(pgraster, FALSE);
+	if (!raster) {
+		PG_FREE_IF_COPY(pgraster, 0);
+		elog(ERROR, "RASTER_setBandPath: Cannot deserialize raster");
+		PG_RETURN_NULL();
+	}
+
+	/* Check index is not NULL or smaller than 1 */
+	if (!PG_ARGISNULL(1))
+		bandindex = PG_GETARG_INT32(1);
+
+	if (bandindex < 1)
+		elog(NOTICE, "Invalid band index (must use 1-based). Returning original raster");
+	else {
+		/* Fetch requested band */
+		band = rt_raster_get_band(raster, bandindex - 1);
+
+		if (!band)
+			elog(NOTICE, "Cannot find raster band of index %d. Returning original raster", bandindex);
+		else if (!rt_band_is_offline(band)) {
+			elog(NOTICE, "Band of index %d is not out-db. Returning original raster", bandindex);
+		}
+		else {
+			/* outdbpath */
+			if (!PG_ARGISNULL(2))
+				outdbpathchar = text_to_cstring(PG_GETARG_TEXT_P(2));
+			else
+				outdbpathchar = rt_band_get_ext_path(band);
+
+			/* outdbindex, is 1-based */
+			if (!PG_ARGISNULL(3))
+			outdbindex = PG_GETARG_INT32(3);
+
+			/* force */
+			if (!PG_ARGISNULL(4))
+				forceset = PG_GETARG_BOOL(4);
+
+			hasnodata = rt_band_get_hasnodata_flag(band);
+			if (hasnodata)
+				rt_band_get_nodata(band, &nodataval);
+
+			newband = rt_band_new_offline_from_path(
+				rt_raster_get_width(raster),
+				rt_raster_get_height(raster),
+				hasnodata,
+				nodataval,
+				outdbindex,
+				outdbpathchar,
+				forceset
+			);
+
+			if (rt_raster_replace_band(raster, newband, bandindex - 1) == NULL)
+				elog(NOTICE, "Cannot change path of band. Returning original raster");
+			else
+				/* old band is in the variable band */
+				rt_band_destroy(band);
+		}
+	}
+
+	/* Serialize raster again */
+	pgrtn = rt_raster_serialize(raster);
+	rt_raster_destroy(raster);
+	PG_FREE_IF_COPY(pgraster, 0);
+	if (!pgrtn) PG_RETURN_NULL();
+
+	SET_VARSIZE(pgrtn, pgrtn->size);
+	PG_RETURN_POINTER(pgrtn);
+}
