@@ -2,7 +2,7 @@
 #!/bin/sh
 
 usage() {
-  echo "Usage: $0"
+  echo "Usage: $0 [<sourcedir>]"
 }
 
 TMPDIR=/tmp/postgis_check_tests_enabled_$$
@@ -25,14 +25,19 @@ check_enabled() {
   #echo "Suffix: ${suffix}"
   #echo "Basedir: ${bd}"
 
-  grep 'topsrcdir)' ${mk} |
-      sed 's|.*topsrcdir)/||;s/ .*$//' |
+  grep 'top_srcdir)' ${mk} |
+      sed 's|.*top_srcdir)/||;s/ .*$//' |
       sed 's|\\||' |
-      sed 's|\.sql\>||' > ${TMPDIR}/enabled_tests
+      sed 's/\.sql$//' > ${TMPDIR}/enabled_tests
+
+  #cat ${TMPDIR}/enabled_tests
 
   find ${bd} -name '*_expected' |
+    sed 's|//|/|' |
     sed 's|_expected$||' > ${TMPDIR}/available_tests
 
+  #cat ${TMPDIR}/available_tests
+
   MISSING=`grep -vwf ${TMPDIR}/enabled_tests ${TMPDIR}/available_tests`
   if test -n "${MISSING}"; then
     (
@@ -47,6 +52,28 @@ check_enabled() {
   fi
 }
 
+### COMMAND LINE PARSING
+RD= # Root source dir
+while [ $# -gt 0 ]; do
+  if [ "$1" = "--help" ]; then
+    usage
+    exit 0
+  elif [ -z "${RD}" ]; then
+    RD=$1
+  else
+    echo "ERROR: unrecognized extra argument $1" >&2
+    usage >&2
+    exit 1
+  fi
+  shift
+done
+
+
+if [ -z "${RD}" ]; then
+  RD=`dirname $0`/..
+fi
+
+cd ${RD}
 err=0
 check_enabled topology/test/tests.mk regress
 err=$(($err+$?))
@@ -54,7 +81,7 @@ check_enabled regress/loader/tests.mk
 err=$(($err+$?))
 check_enabled regress/dumper/tests.mk
 err=$(($err+$?))
-check_enabled sfcgal/regress/tests.mk
+check_enabled sfcgal/regress/tests.mk.in
 err=$(($err+$?))
 check_enabled regress/core/tests.mk.in
 err=$(($err+$?))
