@@ -762,7 +762,7 @@ BEGIN
         new_ver_int := pg_catalog.string_to_array(
             pg_catalog.regexp_replace(
                 new_scripts,
-                '[^\d.].*',
+                E'[^\\d.].*',
                 ''
             ),
             '.'
@@ -775,7 +775,7 @@ BEGIN
         old_ver_int := pg_catalog.string_to_array(
             pg_catalog.regexp_replace(
                 old_scripts,
-                '[^\d.].*',
+                E'[^\\d.].*',
                 ''
             ),
             '.'
